from setuptools import setup

setup(
    name='Pilxi',
    version='5.7',
    packages=['pilxi','pipslx', "pi620lx", "pi743lx", "LXIpxm78xx"],
    url='',
    license='',
    author='Pickering Interfaces',
    author_email='support@pickeringtest.com',
    description='Python wrapper Library for Pickering LXI ClientBridge driver',
)
